﻿using EduSpider.IServices;
using EduSpider.Model.Entity;
using EduSpider.Model.Extend;
using EduSpider.WebApi.Controllers.Base;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json.Linq;
using System.Collections.Generic;
using VTX.FW.Api;
using VTX.FW.Attr;
using VTX.FW.Helper;
using System.Linq;
using EduSpider.Model.Query;

namespace EduSpider.WebApi.Controllers
{
    /// <summary>
    /// 老师相关控制器
    /// </summary>
    public class TeacherController : BaseController
    {

        /// <summary>
        /// 课程仓储接口
        /// </summary>
        [Autowired]
        public ICourseService CourseService { get; set; }

        /// <summary>
        /// 获取老师课程列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult GetTeacherCourse()
        {
            var userInfo = base.UserInfo;
            var query = new Model.Query.CourseQuery()
            {
                TeacherIds = userInfo.AccountId.ToString(),
                KeyWords = base.ReqParameters.GetString("KeyWords"),
            };

            //全部课程
            List<object> AllCourseList = new();
            //已结课
            List<object> FinishList = new();
            //进行中
            List<object> NotFinishList = new();
            var list = CourseService.GetTeacherCourseList(query).Where(qitem => qitem.CourseStatus == 1 || qitem.CompleteNum == qitem.TotalClassNum).ToList();

            if (list != null && list.Count > 0)
            {
                foreach (var item in list)
                {
                    var obj = new
                    {
                        item.courseId,
                        item.CourseName,
                        item.CoverImg,
                        item.CompleteNum,
                        item.TotalClassNum,
                        item.TeacherName,
                    };
                    AllCourseList.Add(obj);
                    if (item.CompleteNum == item.TotalClassNum)
                    {
                        FinishList.Add(obj);
                    }
                    else
                    {
                        NotFinishList.Add(obj);
                    }
                }
            }
            var result = new { AllCourseList, FinishList, NotFinishList };
            return ApiResult.Success(data: result);
        }

        /// <summary>
        /// 获取课程学员列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult GetCourseStu()
        {
            var query = new Model.Query.CourseQuery()
            {
                CourseId = base.ReqParameters.GetInt("courseId")
            };
            List<object> result = new();
            var list = CourseService.GetStuCourseList(query);
            if (list != null && list.Count > 0)
            {
                foreach (var item in list)
                {
                    result.Add(new
                    {
                        StuId = item.uid,
                        StuName = item.name,
                        StuAccount = item.account
                    });
                }
            }
            return ApiResult.Success(data: result);
        }

        /// <summary>
        /// 获取课程作业列表
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult GetCourseHomeWork()
        {
            var query = new Model.Query.CourseQuery()
            {
                CourseId = base.ReqParameters.GetInt("courseId")
            };
            var list = CourseService.GetCourseHomeWork(query);
            return ApiResult.Success(data: list);
        }

        /// <summary>
        /// 获取课程家庭作业详情
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult GetHomeWorkDetails()
        {
            var query = new Model.Query.CourseQuery()
            {
                HomeWorkId = base.ReqParameters.GetInt("HomeWorkId")
            };
            var list = CourseService.GetHomeWorkDetails(query);
            return ApiResult.Success(data: list);
        }

        /// <summary>
        /// 获取课程配置评价信息
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult GetCourseComment()
        {
            var query = new Model.Query.CourseQuery()
            {
                CourseId = base.ReqParameters.GetInt("CourseId")
            };
            var courseRule = CourseService.GetCourseCommentList(query).FirstOrDefault();
            object result;
            if (courseRule != null)
            {
                result = new
                {
                    courseRule.Id,
                    courseRule.Title,
                    courseRule.CourseId,
                    courseRule.CommentDetails
                };
            }
            else
            {
                var sysList = CourseService.GetSysComment();
                result = new
                {
                    Id = 0,
                    Title = "系统默认配置",
                    CourseId = query.CourseId,
                    CommentDetails = sysList
                };
            }
            return ApiResult.Success(data: result);
        }

        /// <summary>
        /// 新增可以课程评语
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult SetCourseComment()
        {
            var model = new RB_Course_Comment_Extend()
            {
                Id = base.ReqParameters.GetInt("Id"),
                CourseId = base.ReqParameters.GetInt("CourseId"),
                Title = base.ReqParameters.GetString("Title"),
                ShowType = base.ReqParameters.GetInt("ShowType"),
            };
            model.CreateBy = base.BaseUserId;
            model.CreateTime = System.DateTime.Now;
            model.Status = 0;
            model.CommentDetails = new List<RB_Course_CommentDetails_Extend>();
            string commentDetails = base.ReqParameters.GetString("CommentDetails");
            if (!string.IsNullOrWhiteSpace(commentDetails))
            {
                JArray array = JArray.Parse(commentDetails);
                if (array != null && array.Count > 0)
                {
                    foreach (var jItem in array)
                    {
                        var sObj = JObject.Parse(jItem.ToString());
                        var details = new RB_Course_CommentDetails_Extend()
                        {
                            DetailId = sObj.GetInt("DetailId"),
                            CourseCommentId = sObj.GetInt("CourseCommentId"),
                            StartNum = sObj.GetDecimal("StartNum"),
                            EndNum = sObj.GetDecimal("EndNum"),
                            Info = sObj.GetString("Info"),
                            Status = 0,
                            CreateBy = base.BaseUserId,
                            CreateTime = System.DateTime.Now,
                            UpdateBy = base.BaseUserId,
                            UpdateTime = System.DateTime.Now,
                        };
                        model.CommentDetails.Add(details);
                    }
                }
            }
            var flag = CourseService.SetCourseComment(model);
            return flag ? ApiResult.Success() : ApiResult.Failed();
        }

        /// <summary>
        /// 删除课程评语
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult RemoveCourseComment()
        {
            string Ids = base.ReqParameters.GetString("Ids");
            var flag = CourseService.DeleteCourseComment(Ids);
            return flag ? ApiResult.Success() : ApiResult.Failed();
        }

        /// <summary>
        /// 根据编号删除课程评价详情配置
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        [HttpGet]
        public ApiResult RemoveCourseCommentDetail()
        {
            string DetailIds = base.ReqParameters.GetString("DetailIds");
            var flag = CourseService.DeleteCourseCommentDetail(DetailIds);
            return flag ? ApiResult.Success() : ApiResult.Failed();
        }


        [HttpPost]
        [HttpGet]
        /// <summary>
        /// 新增修改学员评价
        /// </summary>
        /// <returns></returns>
        public ApiResult SetStuComment()
        {
            var model = new RB_Stu_Comment()
            {
                Id = base.ReqParameters.GetInt("Id"),
                HomeWorkId = base.ReqParameters.GetInt("HomeWorkId"),
                StuUid = base.ReqParameters.GetInt("StuUid"),
                CourseId = base.ReqParameters.GetInt("CourseId"),
                Times = base.ReqParameters.GetInt("Times"),
                Info = base.ReqParameters.GetString("Info"),
                CreateType = base.ReqParameters.GetInt("CreateType"),
            };
            model.ShowType = 3;
            model.Status = 0;
            model.CreateTime = System.DateTime.Now;
            model.CreateBy = base.BaseUserId;
            model.CreateByName = base.UserInfo.AccountName;
            var flag = CourseService.SetStuComment(model);
            return flag ? ApiResult.Success() : ApiResult.Failed();
        }


        [HttpPost]
        [HttpGet]
        /// <summary>
        /// 修改学员评论状态
        /// </summary>
        /// <returns></returns>
        public ApiResult SetStuCommentShowType()
        {
            string Ids = base.ReqParameters.GetString("Ids");
            int ShowType = base.ReqParameters.GetInt("ShowType");
            var flag = CourseService.SetStuCommentShowType(ShowType, Ids);
            return flag ? ApiResult.Success() : ApiResult.Failed();
        }


        [HttpPost]
        [HttpGet]
        /// <summary>
        /// 设置学员评论可见性
        /// </summary>
        /// <returns></returns>
        public ApiResult SetStuCommentShowTypeByTimes()
        {
            int CourseId = base.ReqParameters.GetInt("CourseId");
            int Times = base.ReqParameters.GetInt("Times");
            int ShowType = base.ReqParameters.GetInt("ShowType");
            var flag = CourseService.SetStuCommentShowTypeByTimes(CourseId, Times, ShowType); ;
            return flag ? ApiResult.Success() : ApiResult.Failed();
        }


        [HttpPost]
        [HttpGet]
        /// <summary>
        /// 获取课程已经生成的评论统计列表
        /// </summary>
        /// <returns></returns>
        public ApiResult GetCourseCommentTimes()
        {
            var query = new CourseQuery()
            {
                CourseId = base.ReqParameters.GetInt("CourseId"),
                CreateCommentType = 1
            };
            List<object> result = new();
            var list = CourseService.GetCourseCommentTimesList(query);
            if (list != null && list.Count > 0)
            {
                foreach (var item in list)
                {
                    string ShowTypeStr = "";
                    if (item.ShowType == 1)
                    {
                        ShowTypeStr = "全部可见";
                    }
                    else if (item.ShowType == 2)
                    {
                        ShowTypeStr = "部分可见";
                    }
                    else if (item.ShowType == 3)
                    {
                        ShowTypeStr = "不可见";
                    }
                    result.Add(new
                    {
                        item.Times,
                        item.CourseId,
                        item.CreateType,
                        CreateTypeStr = item.CreateType == 1 ? "系统创建" : "老师创建",
                        item.ShowType,
                        ShowTypeStr,
                        CreateTime = VTX.FW.Helper.ConvertHelper.FormatDate(item.CreateTime),
                        optionsShow = false,
                    });
                }
            }
            return ApiResult.Success(data: result);
        }

        [HttpPost]
        [HttpGet]
        /// <summary>
        /// 获取学员评论列表
        /// </summary>
        /// <returns></returns>
        public ApiResult GetStuCommentList()
        {
            var query = new CourseQuery()
            {
                CourseId = base.ReqParameters.GetInt("CourseId"),
                CommentTimes = base.ReqParameters.GetInt("CommentTimes")
            };
            List<object> result = new();
            var list = CourseService.GetStuCommentList(query);
            if (list.Any())
            {
                foreach (var item in list)
                {
                    string ShowTypeStr = "";
                    if (item.ShowType == 1)
                    {
                        ShowTypeStr = "全部可见";
                    }
                    else if (item.ShowType == 2)
                    {
                        ShowTypeStr = "部分可见";
                    }
                    else if (item.ShowType == 3)
                    {
                        ShowTypeStr = "不可见";
                    }
                    result.Add(new
                    {
                        item.Id,
                        item.HomeWorkId,
                        item.StuUid,
                        item.StuName,
                        item.CourseId,
                        item.Times,
                        item.Info,
                        item.CreateType,
                        CreateTypeStr = item.CreateType == 1 ? "系统创建" : "老师创建",
                        item.ShowType,
                        ShowTypeStr,
                        CreateTime = VTX.FW.Helper.ConvertHelper.FormatDate(item.CreateTime)
                    });
                }
            }
            return ApiResult.Success(data: result);
        }


        [HttpPost]
        [HttpGet]
        /// <summary>
        /// 删除学生评论
        /// </summary>
        /// <returns></returns>
        public ApiResult RemoveStuComment()
        {
            string Ids = base.ReqParameters.GetString("Ids");
            if (string.IsNullOrEmpty(Ids))
            {
                return ApiResult.ParamIsNull(message: "请选择要删除的评论!");
            }
            var list = CourseService.RemoveStuComment(Ids);
            return ApiResult.Success(data: list);
        }

        [HttpPost]
        [HttpGet]
        /// <summary>
        /// 生成课程评论
        /// </summary>
        /// <returns></returns>
        public ApiResult CreateCourseComment()
        {
            int CourseId = base.ReqParameters.GetInt("CourseId");
            CourseService.CreateHomeWorkComment(CourseId);
            return ApiResult.Success();
        }
    }
}
