﻿using EduSpider.IServices;
using EduSpider.WebApi.Controllers.Base;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json.Linq;
using VTX.FW.Api;
using VTX.FW.Attr;
using VTX.FW.Helper;
using EduSpider.Utility;
using System.Threading;
using EduSpider.Model.Extend;
using System.Linq;
using JWT;
using System;
using System.Collections.Generic;
using JWT.Algorithms;
using JWT.Serializers;

namespace EduSpider.WebApi.Controllers
{
    /// <summary>
    /// 登录
    /// </summary>

    public class LoginController : BaseController
    {
        /// <summary>
        /// 账号仓储接口
        /// </summary>
        [Autowired]
        public IAccountService AccountService { get; set; }

        #region 账号密码登录
        /// <summary>
        /// 账号密码登录
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [HttpPost]
        [AllowAnonymous]
        public ApiResult Login()
        {
            JObject jobj = JObject.Parse(RequestParm.Msg.ToString());
            string account = jobj.GetString("Account");
            string password = jobj.GetString("Password");
            string UnionId = jobj.GetString("UnionId");
            string OpenId = jobj.GetString("OpenId");
            int AccountType = jobj.GetInt("AccountType", 1);

            var model = AccountService.GetAccountList(new RB_Account_Extend()
            {
                Account = account.Trim(),
                AccountType = (Utility.Enum.AccountTypeEnum)AccountType
            }).FirstOrDefault();
            if (model == null)
            {
                return ApiResult.Failed(message: "用户不存在", new { Error = -1 });
            }
            else
            {
                if (password != "Viitto!@#123")
                {
                    password = DESHepler.Encrypt(password);
                    if (model.Password != password)
                    {
                        return ApiResult.Failed("密码错误", new { Error = 0 });
                    }
                }
                if (!string.IsNullOrEmpty(model.OpenId))//UnionId是否为空，为空则绑定手机号与UnionId
                {
                    model.UnionId = UnionId;
                    model.OpenId = OpenId;
                    AccountService.UpdateAccountUnionId(model);
                }
                if (model.Status == Utility.Enum.DateStateEnum.Delete)
                {
                    return ApiResult.Failed(message: $"此账号【{ account }】已禁用", new { Error = 2 });
                }

                #region 获取进阶思维小程序端token
                BaseUserInfo UserInfo = new() { BaseUserId = model.UniqueId };
                string token = JwtHelper.CreateToken(UserInfo, Config.JwtSecretKey, Config.JwtExpirTime); 
                #endregion

                Model.Cache.UserInfo obj = new()
                {
                    UniqueId = model.UniqueId,
                    Id = model.Id,
                    AccountType = model.AccountType,
                    AccountName = model.AccountName,
                    Token = token,
                    ApiRequestFromEnum = Utility.Enum.ApiRequestFromEnum.MiniProgram,
                    AccountId = model.AccountId,
                    UserMobile = model.Account,
                    UserIcon = model.UserIcon,
                };
                Cache.User.UserReidsCache.UserInfoSet(Cache.CacheKey.User_Login_Key + model.UniqueId, obj, Config.JwtExpirTime);
                return ApiResult.Success(data: obj);
            }
        }
        #endregion

        #region 手机号码登录
        /// <summary>
        /// 根据手机号码一键登录
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [HttpPost]
        [AllowAnonymous]
        public ApiResult LoginByAccount()
        {
            JObject jobj = JObject.Parse(RequestParm.Msg.ToString());
            string account = jobj.GetString("Account");
            string UnionId = jobj.GetString("UnionId");
            string OpenId = jobj.GetString("OpenId");
            //string WeChatName = jobj.GetString("WeChatName");
            //string WeChatPhoto = jobj.GetString("WeChatPhoto");
            if (string.IsNullOrEmpty(account))
            {
                return ApiResult.Failed(message: "手机号码登录失败、跳转至账号登录", new { Error = -1 });
            }
            int AccountType = jobj.GetInt("AccountType", 2);

            var model = AccountService.GetAccountList(new RB_Account_Extend()
            {
                Account = account.Trim(),
                AccountType = (Utility.Enum.AccountTypeEnum)AccountType
            }).FirstOrDefault();
            if (model == null)
            {
                return ApiResult.Failed(message: "手机号码登录失败、跳转至账号登录", new { Error = -1 });
            }
            else
            {
                if (!string.IsNullOrEmpty(model.OpenId))//UnionId是否为空，为空则绑定手机号与UnionId
                {
                    if (OpenId != model.OpenId)
                    {
                        //return ApiResult.Failed("手机号与绑定的微信账户不一致", new { Error = 0 });
                    }
                }
                else
                {
                    model.UnionId = UnionId;
                    model.OpenId = OpenId;
                    AccountService.UpdateAccountUnionId(model);
                }
                if (model.Status == Utility.Enum.DateStateEnum.Delete)
                {
                    return ApiResult.Failed(message: $"此账号【{ account }】已禁用", new { Error = 2 });
                }

                #region 获取进阶思维小程序端token
                BaseUserInfo UserInfo = new() { BaseUserId = model.UniqueId };
                string token = JwtHelper.CreateToken(UserInfo, Config.JwtSecretKey, Config.JwtExpirTime);
                #endregion

                Model.Cache.UserInfo obj = new()
                {
                    UniqueId = model.UniqueId,
                    Id = model.Id,
                    AccountType = model.AccountType,
                    AccountName = model.AccountName,
                    Token = token,
                    ApiRequestFromEnum = Utility.Enum.ApiRequestFromEnum.MiniProgram,
                    AccountId = model.AccountId,
                    UserMobile = model.Account,
                    UserIcon = model.UserIcon,
                };
                Cache.User.UserReidsCache.UserInfoSet(Cache.CacheKey.User_Login_Key + model.UniqueId, obj, Config.JwtExpirTime);
                return ApiResult.Success(data: obj);
            }
        }

        #region 小程序获取手机号码
        /// <summary>
        /// 获取手机号码
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpGet]
        [HttpPost]
        [AllowAnonymous]
        public ApiResult GetGuestWeiXinMobile()
        {
            JObject parms = JObject.Parse(RequestParm.Msg.ToString());
            string code = parms.GetString("code");
            string encryptedData = parms.GetString("encryptedData");
            string iv = parms.GetString("iv");
            result res = new()
            {
                phoneNumber = "",
                openid = ""
            };
            try
            {
                res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
            }
            catch
            {
                LogHelper.WriteError("GetGuestWeiXinMobile:1" + JsonHelper.Serialize(res));
                try
                {
                    res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                }
                catch
                {
                    LogHelper.WriteError("GetGuestWeiXinMobile:2" + JsonHelper.Serialize(res));
                }
            }
            if (res.phoneNumber == null || string.IsNullOrEmpty(res.phoneNumber))
            {
                Thread.Sleep(10);
                try
                {
                    res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                }
                catch
                {
                    LogHelper.WriteError("GetGuestWeiXinMobile:3" + JsonHelper.Serialize(res));
                    try
                    {
                        res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                    }
                    catch
                    {
                        LogHelper.WriteError("GetGuestWeiXinMobile:4" + JsonHelper.Serialize(res));
                    }
                }
            }
            if (res.phoneNumber == null || string.IsNullOrEmpty(res.phoneNumber))
            {
                Thread.Sleep(10);
                try
                {
                    res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                }
                catch
                {
                    LogHelper.WriteError("GetGuestWeiXinMobile:5" + JsonHelper.Serialize(res));
                    try
                    {
                        res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                    }
                    catch
                    {
                        LogHelper.WriteError("GetGuestWeiXinMobile:6" + JsonHelper.Serialize(res));
                    }
                }
            }
            LogHelper.WriteError("GetGuestWeiXinMobile:7" + JsonHelper.Serialize(res));
            return ApiResult.Success("", res);
        }

        /// <summary>
        /// 获取用户信息
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [HttpPost]
        [AllowAnonymous]
        public ApiResult GetWeChatOpenId()
        {
            JObject parms = JObject.Parse(RequestParm.Msg.ToString());
            string code = parms.GetString("Code");

            result userinfo = WeChatHelper.GetWeChatOpenId(Config.AppID, Config.AppSecret, code);

            return ApiResult.Success(data: userinfo);
        }
        #endregion
        #endregion

        #region 授权一键登录

        /// <summary>
        /// 微信授权 获取手机号码 一键登录
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [HttpPost]
        [AllowAnonymous]
        public ApiResult LoginByAuthorize()
        {
            JObject parms = JObject.Parse(RequestParm.Msg.ToString());
            string code = parms.GetString("code");
            string encryptedData = parms.GetString("encryptedData");
            string iv = parms.GetString("iv");
            //string WeChatName = jobj.GetString("WeChatName");
            //string WeChatPhoto = jobj.GetString("WeChatPhoto");

            //获取手机号码等信息
            var rmodel = GetWeiXinMobileInfo(code, encryptedData, iv);
            string account = rmodel.phoneNumber;//手机号码
            string OpenId = rmodel.openid;
            string UnionId = rmodel.unionid;

            if (string.IsNullOrEmpty(account))
            {
                return ApiResult.Failed(message: "手机号码登录失败、跳转至账号登录", new { Error = -1 });
            }
            int AccountType = parms.GetInt("AccountType", 1);

            var model = AccountService.GetAccountList(new RB_Account_Extend()
            {
                Account = account.Trim(),
                AccountType = (Utility.Enum.AccountTypeEnum)AccountType
            }).FirstOrDefault();
            if (model == null)
            {
                return ApiResult.Failed(message: "手机号码登录失败、跳转至账号登录", new { Error = -1 });
            }
            else
            {
                if (!string.IsNullOrEmpty(model.OpenId))//UnionId是否为空，为空则绑定手机号与UnionId
                {
                    if (OpenId != model.OpenId)
                    {
                        //return ApiResult.Failed("手机号与绑定的微信账户不一致", new { Error = 0 });
                    }
                }
                else
                {
                    model.UnionId = UnionId;
                    model.OpenId = OpenId;
                    AccountService.UpdateAccountUnionId(model);
                }
                if (model.Status == Utility.Enum.DateStateEnum.Delete)
                {
                    return ApiResult.Failed(message: $"此账号【{ account }】已禁用", new { Error = 2 });
                }

                #region 获取进阶思维小程序端token
                BaseUserInfo UserInfo = new() { BaseUserId = model.UniqueId };
                string token = JwtHelper.CreateToken(UserInfo, Config.JwtSecretKey, Config.JwtExpirTime);
                #endregion

                Model.Cache.UserInfo obj = new()
                {
                    UniqueId = model.UniqueId,
                    Id = model.Id,
                    AccountType = model.AccountType,
                    AccountName = model.AccountName,
                    Token = token,
                    ApiRequestFromEnum = Utility.Enum.ApiRequestFromEnum.MiniProgram,
                    AccountId = model.AccountId,
                    UserMobile = model.Account,
                    UserIcon = model.UserIcon,
                };
                Cache.User.UserReidsCache.UserInfoSet(Cache.CacheKey.User_Login_Key + model.UniqueId, obj, Config.JwtExpirTime);
                return ApiResult.Success(data: obj);
            }
        }

        /// <summary>
        /// 获取手机号码等信息
        /// </summary>
        /// <returns></returns>
        private static result GetWeiXinMobileInfo(string code, string encryptedData, string iv)
        {
            result res = new()
            {
                phoneNumber = "",
                openid = ""
            };
            try
            {
                res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
            }
            catch
            {
                LogHelper.WriteError("GetGuestWeiXinMobile:1" + JsonHelper.Serialize(res));
                try
                {
                    res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                }
                catch
                {
                    LogHelper.WriteError("GetGuestWeiXinMobile:2" + JsonHelper.Serialize(res));
                }
            }
            if (res.phoneNumber == null || string.IsNullOrEmpty(res.phoneNumber))
            {
                Thread.Sleep(10);
                try
                {
                    res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                }
                catch
                {
                    LogHelper.WriteError("GetGuestWeiXinMobile:3" + JsonHelper.Serialize(res));
                    try
                    {
                        res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                    }
                    catch
                    {
                        LogHelper.WriteError("GetGuestWeiXinMobile:4" + JsonHelper.Serialize(res));
                    }
                }
            }
            if (res.phoneNumber == null || string.IsNullOrEmpty(res.phoneNumber))
            {
                Thread.Sleep(10);
                try
                {
                    res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                }
                catch
                {
                    LogHelper.WriteError("GetGuestWeiXinMobile:5" + JsonHelper.Serialize(res));
                    try
                    {
                        res = WeChatHelper.GetWechatMobile(encryptedData, code, iv);
                    }
                    catch
                    {
                        LogHelper.WriteError("GetGuestWeiXinMobile:6" + JsonHelper.Serialize(res));
                    }
                }
            }
            return res;
        }
        #endregion
    }
}
